"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.help = exports.commandNames = exports.cliOptionsTypes = exports.rcOptionsTypes = void 0;
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const error_1 = __importDefault(require("@pnpm/error"));
const fetch_1 = require("@pnpm/fetch");
const node_resolver_1 = require("@pnpm/node.resolver");
const cmd_shim_1 = __importDefault(require("@zkochan/cmd-shim"));
const render_help_1 = __importDefault(require("render-help"));
const node_1 = require("./node");
const getNodeMirror_1 = __importDefault(require("./getNodeMirror"));
const parseNodeEditionSpecifier_1 = require("./parseNodeEditionSpecifier");
function rcOptionsTypes() {
    return {};
}
exports.rcOptionsTypes = rcOptionsTypes;
function cliOptionsTypes() {
    return {
        global: Boolean,
    };
}
exports.cliOptionsTypes = cliOptionsTypes;
exports.commandNames = ['env'];
function help() {
    return (0, render_help_1.default)({
        description: 'Install and use the specified version of Node.js. The npm CLI bundled with the given Node.js version gets installed as well.',
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    {
                        description: 'Installs Node.js globally',
                        name: '--global',
                        shortAlias: '-g',
                    },
                ],
            },
        ],
        url: (0, cli_utils_1.docsUrl)('env'),
        usages: [
            'pnpm env use --global <version>',
            'pnpm env use --global 16',
            'pnpm env use --global lts',
            'pnpm env use --global argon',
            'pnpm env use --global latest',
            'pnpm env use --global rc/16',
        ],
    });
}
exports.help = help;
async function handler(opts, params) {
    if (params.length === 0) {
        throw new error_1.default('ENV_NO_SUBCOMMAND', 'Please specify the subcommand');
    }
    switch (params[0]) {
        case 'use': {
            if (!opts.global) {
                throw new error_1.default('NOT_IMPLEMENTED_YET', '"pnpm env use <version>" can only be used with the "--global" option currently');
            }
            const fetch = (0, fetch_1.createFetchFromRegistry)(opts);
            const { releaseChannel, versionSpecifier } = (0, parseNodeEditionSpecifier_1.parseNodeEditionSpecifier)(params[1]);
            const nodeMirrorBaseUrl = (0, getNodeMirror_1.default)(opts.rawConfig, releaseChannel);
            const nodeVersion = await (0, node_resolver_1.resolveNodeVersion)(fetch, versionSpecifier, nodeMirrorBaseUrl);
            if (!nodeVersion) {
                throw new error_1.default('COULD_NOT_RESOLVE_NODEJS', `Couldn't find Node.js version matching ${params[1]}`);
            }
            const nodeDir = await (0, node_1.getNodeDir)(fetch, {
                ...opts,
                useNodeVersion: nodeVersion,
                nodeMirrorBaseUrl,
            });
            const src = path_1.default.join(nodeDir, process.platform === 'win32' ? 'node.exe' : 'bin/node');
            const dest = path_1.default.join(opts.bin, process.platform === 'win32' ? 'node.exe' : 'node');
            try {
                await fs_1.promises.unlink(dest);
            }
            catch (err) { }
            await fs_1.promises.symlink(src, dest, 'file');
            try {
                let npmDir = nodeDir;
                if (process.platform !== 'win32') {
                    npmDir = path_1.default.join(npmDir, 'lib');
                }
                npmDir = path_1.default.join(npmDir, 'node_modules/npm');
                if (opts.configDir) {
                    // We want the global npm settings to persist when Node.js or/and npm is changed to a different version,
                    // so we tell npm to read the global config from centralized place that is outside of npm's directory.
                    await fs_1.promises.writeFile(path_1.default.join(npmDir, 'npmrc'), `globalconfig=${path_1.default.join(opts.configDir, 'npmrc')}`, 'utf-8');
                }
                const npmBinDir = path_1.default.join(npmDir, 'bin');
                const cmdShimOpts = { createPwshFile: false };
                await (0, cmd_shim_1.default)(path_1.default.join(npmBinDir, 'npm-cli.js'), path_1.default.join(opts.bin, 'npm'), cmdShimOpts);
                await (0, cmd_shim_1.default)(path_1.default.join(npmBinDir, 'npx-cli.js'), path_1.default.join(opts.bin, 'npx'), cmdShimOpts);
            }
            catch (err) { // eslint-disable-line
                // ignore
            }
            return `Node.js ${nodeVersion} is activated
  ${dest} -> ${src}`;
        }
        default: {
            throw new error_1.default('ENV_UNKNOWN_SUBCOMMAND', 'This subcommand is not known');
        }
    }
}
exports.handler = handler;
//# sourceMappingURL=env.js.map